<?php

if (!defined('ABSPATH')) {
    exit;
}

class EmoSMConnectorConnectorSettingsPage
{
    private $options;

    public function __construct()
    {
        add_action('admin_menu', array($this, 'addPluginPage'));
        add_action('admin_init', array($this, 'pageInit'));
    }
    
    /** Add options page */
    public function addPluginPage()
    {
        // This page will be under "Settings"
        add_menu_page(
            'Store Manager Connector',
            'Store Manager Connector',
            'manage_options',
            'smconnector',
            array($this, 'createAdminPage'),
            plugins_url('/images/woo.png', __FILE__)
        );
    }

    /**
     * Options page callback
     */
    public function createAdminPage()
    {
        global $wpdb;

        $this->options = get_option('SMCONNECTOR_OPTIONS');

        if (!$this->options)
            $wpdb->replace(
                $wpdb->options,
                array('option_name' => 'SMCONNECTOR_OPTIONS', 'option_value' => serialize(emo_smc_get_default_smconnector_options()))
            );
    ?>
        <div class="wrap">
            <h2>Store Manager Connector</h2>
            <form method="post" action="options.php">
            <?php settings_fields('smconnector_group'); ?>
			<div class="section_wrap">
			<?php do_settings_sections('smconnector-access'); ?>
			</div>
			<div class="section_wrap">
			<?php do_settings_sections('smconnector-smconnectoroptions'); ?>
			</div>
			<div class="button_toolbar tablenav bottom">
            <?php submit_button(__('Save Settings', 'store-manager-connector'), 'primary', 'submit-form', false); ?>
			</div>
            </form>
        </div>
        <?php
    }
    
    /** Register and add settings */
    public function pageInit()
    {
        register_setting(
            'smconnector_group', // Option group
            'SMCONNECTOR_OPTIONS', // Option name
            array($this, 'sanitize') // Sanitize
        );

        // Add Access Settings
        add_settings_section(
            'setting_section_id', // ID
            __('Access Settings', 'store-manager-connector'), // Title
            array($this, 'printSectionAccess'),
            'smconnector-access' // Page
        );

        // Add Store Manager Connector Options Settings
        add_settings_section(
            'setting_section_id', // ID
            __('Store Manager Connector Settings', 'store-manager-connector'), // Title
            array($this, 'printSectionSMConnectorOptions'),
            'smconnector-smconnectoroptions' // Page
        );

        // Add field 'login'
        add_settings_field(
            'smconnector_login', // ID
            __('Login', 'store-manager-connector'), // Title
            array($this, 'loginCallback'), // Callback
            'smconnector-access', // Page
            'setting_section_id' // Section
        );

        // Add field 'password'
        add_settings_field(
            'smconnector_password',
            __('Password', 'store-manager-connector'),
            array($this, 'passwordCallback'),
            'smconnector-access',
            'setting_section_id'
        );

        // Add field 'tmp_dir'
        add_settings_field(
            'smconnector_tmp_dir',
            __('Directory for Module Operations', 'store-manager-connector'),
            array($this, 'tmpDirCallback'),
            'smconnector-smconnectoroptions',
            'setting_section_id'
        );

        // Add field 'allow_compression'
        add_settings_field(
            'smconnector_allow_compression',
            __('Allow Compression', 'store-manager-connector'),
            array($this, 'allowCompressionCallback'),
            'smconnector-smconnectoroptions',
            'setting_section_id'
        );

        // Add field 'compress_level'
        add_settings_field(
            'smconnector_compress_level',
            __('Compress Level', 'store-manager-connector'),
            array($this, 'compressLevelCallback'),
            'smconnector-smconnectoroptions',
            'setting_section_id'
        );

        // Add field 'limit_query_size'
        add_settings_field(
            'smconnector_limit_query_size',
            __('Limit Query Size', 'store-manager-connector'),
            array($this, 'limitQuerySizeCallback'),
            'smconnector-smconnectoroptions',
            'setting_section_id'
        );

        // Add field 'package_size'
        add_settings_field(
            'smconnector_package_size',
            __('Package Size', 'store-manager-connector'),
            array($this, 'packageSizeCallback'),
            'smconnector-smconnectoroptions',
            'setting_section_id'
        );

        // Add field 'exclude_db_tables'
        add_settings_field(
            'smconnector_exclude_db_tables',
            __('Exclude Tables', 'store-manager-connector'),
            array($this, 'excludeDbTablesCallback'),
            'smconnector-smconnectoroptions',
            'setting_section_id'
        );

        // Add field 'allowed_ips'
        add_settings_field(
            'smconnector_allowed_ips',
            __('Allowed IPs', 'store-manager-connector'),
            array($this, 'allowedIpsCallback'),
            'smconnector-smconnectoroptions',
            'setting_section_id'
        );

        // Add field 'notification_email'
        /*add_settings_field(
            'smconnector_notification_email',
            'Email for Log Reports',
            array($this, 'notificationEmailCallback'),
            'smconnector-smconnectoroptions',
            'setting_section_id'
        );*/
    }

    /**
     * Sanitize each setting field as needed
     *
     * @param array $input Contains all settings fields as array keys
     * @return array
     */
    public function sanitize($input)
    {
        $new_options = array();
        $options_prev = get_option('SMCONNECTOR_OPTIONS');
        $password_prev = isset($options_prev['password'])
            ? $options_prev['password']
            : emo_smc_get_encrypted_password(EMO_SMC_DEFAULT_PASSWORD);

        $new_options['login'] = isset($input['smconnector_login'])
            ? sanitize_text_field($input['smconnector_login'])
            : EMO_SMC_DEFAULT_LOGIN;
        $new_options['password'] = !empty($input['smconnector_password']) && $input['smconnector_password'] != $password_prev
                ? $input['smconnector_password']
                : EMO_SMC_DEFAULT_PASSWORD;
        $new_options['allow_compression'] = isset($input['smconnector_allow_compression'])
            ? (int)$input['smconnector_allow_compression']
            : 0;
        $new_options['exclude_db_tables'] = isset($input['smconnector_exclude_db_tables'])
            ? implode(';', $input['smconnector_exclude_db_tables'])
            : implode(';', emo_smc_get_default_excluded_tables());
        $new_options['allowed_ips'] = isset($input['smconnector_allowed_ips'])
            ? (string)$input['smconnector_allowed_ips']
            : EMO_SMC_DEFAULT_ALLOWED_IPS;
        /*$new_options['notification_email'] = isset($input['smconnector_notification_email'])
            ? (string)$input['smconnector_notification_email']
            : DEFAULT_NOTIFICATION_EMAIL;*/

        if (empty($input['smconnector_password'])) {
            $new_options['password'] = EMO_SMC_DEFAULT_PASSWORD;
        } elseif ($input['smconnector_password'] == $password_prev) {
            $new_options['password'] = emo_smc_get_decrypted_password($password_prev);
        } else {
            $new_options['password'] = (string)$input['smconnector_password'];
        }

        if (isset($input['smconnector_tmp_dir'])) {
            // trim '/', '\', '.' from begin and end
            $new_options['tmp_dir'] = preg_replace(
                '/^[\/|\\\|\.]*|[\/|\\\|\.]*$/',
                '',
                $input['smconnector_tmp_dir']
            );

            // add '/' to begin
            $new_options['tmp_dir'] = '/' . $new_options['tmp_dir'];
        } else {
            $new_options['tmp_dir'] = emo_smc_get_tmp_path();
        }

        if (isset($input['smconnector_compress_level'])) {
            $new_options['compress_level'] = (int)$input['smconnector_compress_level'];

            if ($new_options['compress_level'] < EMO_SMC_MIN_COMPRESS_LEVEL) {
                $new_options['compress_level'] = EMO_SMC_MIN_COMPRESS_LEVEL;
            } elseif ($new_options['compress_level'] > EMO_SMC_MAX_COMPRESS_LEVEL) {
                $new_options['compress_level'] = EMO_SMC_MAX_COMPRESS_LEVEL;
            }
        } else {
            $new_options['compress_level'] = EMO_SMC_DEFAULT_COMPRESS_LEVEL;
        }

        if (isset($input['smconnector_limit_query_size'])) {
            $new_options['limit_query_size'] = (int)$input['smconnector_limit_query_size'];

            if ($new_options['limit_query_size'] < EMO_SMC_MIN_LIMIT_QUERY_SIZE) {
                $new_options['limit_query_size'] = EMO_SMC_MIN_LIMIT_QUERY_SIZE;
            } elseif ($new_options['limit_query_size'] > EMO_SMC_MAX_LIMIT_QUERY_SIZE) {
                $new_options['limit_query_size'] = EMO_SMC_MAX_LIMIT_QUERY_SIZE;
            }
        } else {
            $new_options['limit_query_size'] = EMO_SMC_DEFAULT_LIMIT_QUERY_SIZE;
        }

        if (isset($input['smconnector_package_size'])) {
            $new_options['package_size'] = (int)$input['smconnector_package_size'];

            if ($new_options['package_size'] < EMO_SMC_MIN_PACKAGE_SIZE) {
                $new_options['package_size'] = EMO_SMC_MIN_PACKAGE_SIZE;
            } elseif ($new_options['package_size'] > EMO_SMC_MAX_PACKAGE_SIZE) {
                $new_options['package_size'] = EMO_SMC_MAX_PACKAGE_SIZE;
            }
        } else {
            $new_options['package_size'] = EMO_SMC_DEFAULT_PACKAGE_SIZE;
        }

        $new_options['smconnector_hash'] = md5($new_options['login'] . $new_options['password']);
        $new_options['password'] = emo_smc_get_encrypted_password($new_options['password']);

        return $new_options;
    }
    
    /**
     * Print the Access Section text
     */
    public function printSectionAccess()
    {
        echo __('Enter Store Manager Connector credentials below:', 'store-manager-connector');
    }

    /**
     * Print the Store Manager Connector Options Section text
     */
    public function printSectionSMConnectorOptions()
    {
        echo __('Enter Store Manager Connector settings below:', 'store-manager-connector');
    }

    public function loginCallback()
    {
        if (!isset($this->options['login'])) {
            $this->options['login'] = EMO_SMC_DEFAULT_LOGIN;
        }

        printf(
            '<input type="text" id="smconnector_login" name="SMCONNECTOR_OPTIONS[smconnector_login]" value="%s" /> %s',
            esc_attr($this->options['login']),
            $this->options['login'] == EMO_SMC_DEFAULT_LOGIN
                ? '<span class="warning"><br />' . __('Default login is "1". Change it because of security reasons!', 'store-manager-connector') . '</span>'
                : ''
        );
    }
    
    public function passwordCallback()
    {
        $password_changed = false;

        if (!isset($this->options['password'])) {
            $this->options['password'] = emo_smc_get_encrypted_password(EMO_SMC_DEFAULT_PASSWORD);
        }

        if ($this->options['password'] != emo_smc_get_encrypted_password(EMO_SMC_DEFAULT_PASSWORD)) {
            $password_changed = true;
        }

        printf(
            '<input type="password" id="smconnector_password" name="SMCONNECTOR_OPTIONS[smconnector_password]"
            value="%s" />%s',
            $this->options['password'],
            !$password_changed
                ? '<span class="warning"><br />' . __('Default login is "1". Change it because of security reasons!', 'store-manager-connector') . '</span>'
                : ''
        );
    }

    public function tmpDirCallback()
    {
        $warning = '';

        if (!isset($this->options['tmp_dir'])) {
            $this->options['tmp_dir'] = emo_smc_get_tmp_path();
        }

        if (!is_dir(ABSPATH . $this->options['tmp_dir'])) {
            $warning = __('Directory does not exist', 'store-manager-connector');
        } elseif (!is_writable(ABSPATH . $this->options['tmp_dir'])) {
            $warning = __('Set writing permissions for temporary directory', 'store-manager-connector');
        }

        echo '<input type="text" id="smconnector_tmp_dir" name="SMCONNECTOR_OPTIONS[smconnector_tmp_dir]" value="'
            . esc_attr($this->options['tmp_dir']) . '" /><span class="field_hint"><br />'
            . __('Enter temporary folder path. It should be writable.', 'store-manager-connector') . '</span>';

        if (!empty($warning)) {
            echo '<span class="warning"><br />' . $warning . '</span>';
        }
    }

    public function allowCompressionCallback()
    {
        if (!isset($this->options['allow_compression'])) {
            $this->options['allow_compression'] = EMO_SMC_DEFAULT_ALLOW_COMPRESSION;
        } else {
            $this->options['allow_compression'] = (int)$this->options['allow_compression'];
        }

        printf(
            '<input type="checkbox" id="smconnector_allow_compression"
            name="SMCONNECTOR_OPTIONS[smconnector_allow_compression]" value="1" %s/><span class="field_hint"><br />'
            . __('Compression of generated dump file. It is recommended for save space and faster getting data in Store
            Manager.', 'store-manager-connector') . '</span>',
            ($this->options['allow_compression'] == 1) ? 'checked' : ''
        );
    }

    public function compressLevelCallback()
    {
        if (!isset($this->options['compress_level'])) {
            $this->options['compress_level'] = EMO_SMC_DEFAULT_COMPRESS_LEVEL;
        } elseif ((int)$this->options['compress_level'] < EMO_SMC_MIN_COMPRESS_LEVEL) {
            $this->options['compress_level'] = EMO_SMC_MIN_COMPRESS_LEVEL;
        } elseif ((int)$this->options['compress_level'] > EMO_SMC_MAX_COMPRESS_LEVEL) {
            $this->options['compress_level'] = EMO_SMC_MAX_COMPRESS_LEVEL;
        } else {
            $this->options['compress_level'] = (int)$this->options['compress_level'];
        }

        echo '<input type="text" id="smconnector_compress_level"
            name="SMCONNECTOR_OPTIONS[smconnector_compress_level]" value="' . $this->options['compress_level'] . '" />
            <span class="field_hint"><br />' . __('Values between 1 and 9 will trade off speed and efficiency.
             The 1 flag means fast but less efficient" compression, and 9 means "slow but most efficient" compression.',
               'store-manager-connector') . '</span>';
    }

    public function limitQuerySizeCallback()
    {
        if (!isset($this->options['limit_query_size'])) {
            $this->options['limit_query_size'] = EMO_SMC_DEFAULT_LIMIT_QUERY_SIZE;
        } elseif ((int)$this->options['limit_query_size'] < EMO_SMC_MIN_LIMIT_QUERY_SIZE) {
            $this->options['limit_query_size'] = EMO_SMC_MIN_LIMIT_QUERY_SIZE;
        } elseif ((int)$this->options['limit_query_size'] > EMO_SMC_MAX_LIMIT_QUERY_SIZE) {
            $this->options['limit_query_size'] = EMO_SMC_MAX_LIMIT_QUERY_SIZE;
        } else {
            $this->options['limit_query_size'] = (int)$this->options['limit_query_size'];
        }

        echo '<input type="text" id="smconnector_limit_query_size"
            name="SMCONNECTOR_OPTIONS[smconnector_limit_query_size]" value="' . $this->options['limit_query_size']
            . '" /><span class="field_hint"><br />' . __('Restrict capacity of queries per one request (kB).',
                'store-manager-connector') . '</span>';
    }

    public function packageSizeCallback()
    {
        if (!isset($this->options['package_size'])) {
            $this->options['package_size'] = EMO_SMC_DEFAULT_PACKAGE_SIZE;
        } elseif ((int)$this->options['package_size'] < EMO_SMC_MIN_PACKAGE_SIZE) {
            $this->options['package_size'] = EMO_SMC_MIN_PACKAGE_SIZE;
        } elseif ((int)$this->options['package_size'] > EMO_SMC_MAX_PACKAGE_SIZE) {
            $this->options['package_size'] = EMO_SMC_MAX_PACKAGE_SIZE;
        } else {
            $this->options['package_size'] = (int)$this->options['package_size'];
        }

        echo '<input type="text" id="smconnector_package_size" name="SMCONNECTOR_OPTIONS[smconnector_package_size]"
            value="' . $this->options['package_size'] . '" /><span class="field_hint"><br />'
         . __('Size of parts for getting dump file (kB).', 'store-manager-connector') . '</span>';
    }

    public function excludeDbTablesCallback()
    {
        global $wpdb;

        $options = '<div id="exclude_db_tables_list">';
        $query = 'SHOW TABLES';
        $tables = $wpdb->get_results($query, ARRAY_N);

        if (isset($this->options['exclude_db_tables'])) {
            $excluded_tables = explode(';', $this->options['exclude_db_tables']);
        } else {
            $excluded_tables = emo_smc_get_default_excluded_tables();
        }

        foreach ($tables as $table_name) {
            $checked = in_array($table_name[0], $excluded_tables) ? 'checked' : '';
            $options .= '<label><input type="checkbox" name="SMCONNECTOR_OPTIONS[smconnector_exclude_db_tables][]" value="'
                . $table_name[0] . '" ' . $checked . '>' . $table_name[0] . '</label><br>';
        }

        echo $options;
        echo '</div><span class="field_hint">'
          . __('Do not get data from tables selected here. Use this to reduce size of the data retrieved from module.',
              'store-manager-connector') . '</span>';
    }

    public function allowedIpsCallback()
    {
        if (!isset($this->options['allowed_ips'])) {
            $this->options['allowed_ips'] = EMO_SMC_DEFAULT_ALLOWED_IPS;
        }

        $this->options['allowed_ips'] = (string)$this->options['allowed_ips'];

        echo '<input type="text" id="smconnector_allowed_ips" name="SMCONNECTOR_OPTIONS[smconnector_allowed_ips]"
            value="' . $this->options['allowed_ips'] . '" /><span class="field_hint"><br />'
            . __('In order to allow module using only from specific IP address you should add IP address here
            (for example, 48.78.88.98 - only one IP address; 48.78.88.98, 15.25.35.45 - two IP addresses;
            48.78.x.x - all IP addresses which begin from 48.78.)', 'store-manager-connector') . '</span>';
    }

    /*public function notificationEmailCallback()
    {
        if (!isset($this->options['notification_email'])) {
            $this->options['notification_email'] = DEFAULT_NOTIFICATION_EMAIL;
        }

        $this->options['notification_email'] = (string)$this->options['notification_email'];

        echo '<input type="text" id="smconnector_notification_email" name="SMCONNECTOR_OPTIONS[smconnector_notification_email]" value="' . $this->options['notification_email'] . '" />
            <span class="field_hint"><br />Please enter your email address here to receive notifications.</span>';
    }*/
}

if (is_admin()) {
    $GLOBALS['SMConnectorSettingsPage'] = new EmoSMConnectorConnectorSettingsPage();
}
